ifndef			_CAMERA_ASM
_CAMERA_ASM		equ	1

; ------------------------------------------------------
; Includes
			include	Vertices.asm

; ------------------------------------------------------
; Variables
qHeading		QUAT	<>
qPitch			QUAT	<>

; ------------------------------------------------------
; Name: Cam_Orientation()
; Desc: Set the camera's Pitch & Heading
Cam_Orientation		proc	uses ebx Pitch:real4, Heading:real4
			local	_q:QUAT
			local	_m:MATRIX

			invoke	QuatSetValue, addr qPitch, CFLT(1.0), CFLT(0.0), CFLT(0.0), Pitch
			invoke	QuatSetValue, addr qHeading, CFLT(0.0), CFLT(1.0), CFLT(0.0), Heading
			invoke	QuatMul, addr qPitch, addr qHeading, addr _q
			invoke	QuatInverse, addr _q
			invoke	QuatToMat, addr _q, addr _m
			invoke	glLoadMatrixf, addr _m
			ret
Cam_Orientation		endp

; ------------------------------------------------------
; Name: Quat_RotateX()
; Desc: Rotate a quaternion on X axis
Quat_RotateX		proc	Quat:LPQUAT, RotX:real4
			invoke	QuatSetValue, Quat, CFLT(0.0), CFLT(1.0), CFLT(0.0), RotX
			ret
Quat_RotateX		endp

; ------------------------------------------------------
; Name: Quat_RotateY()
; Desc: Rotate a quaternion on Y axis
Quat_RotateY		proc	Quat:LPQUAT, RotY:real4
			invoke	QuatSetValue, Quat, CFLT(1.0), CFLT(0.0), CFLT(0.0), RotY
			ret
Quat_RotateY		endp

; ------------------------------------------------------
; Name: Quat_RotateZ()
; Desc: Rotate a quaternion on Z axis
Quat_RotateZ		proc	Quat:LPQUAT, RotZ:real4
			invoke	QuatSetValue, Quat, CFLT(0.0), CFLT(0.0), CFLT(1.0), RotZ
			ret
Quat_RotateZ		endp

; ------------------------------------------------------
; Name: Quat_RotateXYZ()
; Desc: Rotate a quaternion on XYZ axises
Quat_RotateXYZ		proc	Quat:LPQUAT, RotX:real4, RotY:real4, RotZ:real4
			local	_RotX:real4
			local	_RotY:real4
			local	_RotZ:real4

			; Convert them into radians
			fld	RotX
			fmul	_FltPI180
			fstp	_RotX
			fld	RotY
			fmul	_FltPI180
			fstp	_RotY
			fld	RotZ
			fmul	_FltPI180
			fstp	_RotZ
			invoke	EulerToQuat, _RotY, _RotX, _RotZ, Quat
			invoke	QuatInverse, Quat
			ret
Quat_RotateXYZ		endp

; ------------------------------------------------------
; Name: Matrix_Translate()
; Desc: Set the positions in a matrix
Matrix_Translate	proc	Matrix:LPMATRIX, PosX:real4, PosY:real4, PosZ:real4
			mov	eax, Matrix
			fld	CFLT(0.0)
			fstp	[eax + MATRIX._41]
			fld	CFLT(0.0)
			fstp	[eax + MATRIX._42]
			fld	CFLT(-4.0)
			fstp	[eax + MATRIX._43]
			ret
Matrix_Translate	endp

endif